@@ -66,8 +66,7 @@ class ceVAE(pl.LightningModule):
         self.model = VAE(
             input_size=input_shape[1:], z_dim=z_dim, fmap_sizes=model_feature_map_sizes)
 
-        # already saved via config
-        # self.save_hyperparameters()
+        self.save_hyperparameters()
 
     def configure_optimizers(self):
         optimizer = optim.Adam(self.model.parameters(), lr=self.lr)
@@ -80,49 +79,57 @@ class ceVAE(pl.LightningModule):
         self.mode = "sample"
 
     def training_step(self, batch, batch_idx):
-        x, _ = batch  # only inputs no labels
+        x, attrs = batch  # only inputs no labels
 
         # VAE Part
         loss_vae = 0
         loss_vae_kl = 0
         loss_vae_rec = 0
-        if self.ce_factor < 1:
-            x_rec_vae, z_dist, = self.model(x)
+        try:
+            if self.ce_factor < 1:
+                x_rec_vae, z_dist, = self.model(x)
 
-            if self.beta > 0:
-                loss_vae_kl = self.kl_loss_fn(z_dist) * self.beta
+                if self.beta > 0:
+                    loss_vae_kl = self.kl_loss_fn(z_dist) * self.beta
 
-            loss_vae_rec = self.rec_loss_fn(x_rec_vae, x)
-            loss_vae = loss_vae_kl + loss_vae_rec * self.theta
+                loss_vae_rec = self.rec_loss_fn(x_rec_vae, x)
+                loss_vae = loss_vae_kl + loss_vae_rec * self.theta
 
-        # CE Part
-        loss_ce = 0
-        if self.ce_factor > 0:
-            ce_tensor = get_square_mask(
-                x.shape,
-                square_size=(0, np.max(self.input_shape[2:]) // 2),
-                noise_val=(torch.min(x).item(),
-                           torch.max(x).item()),
-                n_squares=(0, 3),
-            )
-            ce_tensor = torch.from_numpy(ce_tensor).float()
-            ce_tensor = ce_tensor.type_as(x)
-            inpt_noisy = torch.where(ce_tensor != 0, ce_tensor, x)
-            x_rec_ce, _ = self.model(inpt_noisy)
-            rec_loss_ce = self.rec_loss_fn(x_rec_ce, x)
-            loss_ce = rec_loss_ce
+            # CE Part
+            loss_ce = 0
+            if self.ce_factor > 0:
+                ce_tensor = get_square_mask(
+                    x.shape,
+                    square_size=(0, np.max(self.input_shape[2:]) // 2),
+                    noise_val=(torch.min(x).item(),
+                               torch.max(x).item()),
+                    n_squares=(0, 3),
+                )
+                ce_tensor = torch.from_numpy(ce_tensor).float()
+                ce_tensor = ce_tensor.type_as(x)
+                inpt_noisy = torch.where(ce_tensor != 0, ce_tensor, x)
+                x_rec_ce, _ = self.model(inpt_noisy)
+                rec_loss_ce = self.rec_loss_fn(x_rec_ce, x)
+                loss_ce = rec_loss_ce
 
-        loss = (1.0 - self.ce_factor) * \
-            loss_vae + self.ce_factor * loss_ce
+            loss = (1.0 - self.ce_factor) * \
+                loss_vae + self.ce_factor * loss_ce
 
-        # Generalized ELBO with Constrained Optimization
-        if self.use_geco and self.ce_factor < 1:
-            g_goal = 0.1
-            g_lr = 1e-4
-            self.vae_loss_ema = (1.0 - 0.9) * \
-                loss_vae_rec + 0.9 * self.vae_loss_ema
-            self.theta = self.geco_beta_update(
-                self.theta, self.vae_loss_ema, g_goal, g_lr, speedup=2)
+            # Generalized ELBO with Constrained Optimization
+            if self.use_geco and self.ce_factor < 1:
+                g_goal = 0.1
+                g_lr = 1e-4
+                self.vae_loss_ema = (1.0 - 0.9) * \
+                    loss_vae_rec + 0.9 * self.vae_loss_ema
+                self.theta = self.geco_beta_update(
+                    self.theta, self.vae_loss_ema, g_goal, g_lr, speedup=2)
+
+                self.log('geco_theta', self.theta)
+                self.log('geco_vae_loss_ema', self.vae_loss_ema)
+        except Exception as e:
+            logger.error(
+                f"exception during training step for data {x} with attrs {attrs}", e)
+            raise e
 
         if batch_idx % self.print_every_iter == 0:
             sample_images = []
@@ -143,6 +150,7 @@ class ceVAE(pl.LightningModule):
         self.log('train_loss', loss)
         self.log('train_loss_vae', loss_vae)
         self.log('train_loss_ce', loss_ce)
+
         return {"loss": loss, "loss_vae": loss_vae, "loss_ce": loss_ce, "loss_vae_kl": loss_vae_kl, "loss_vae_rec": loss_vae_rec}
 
     def validation_step(self, batch, batch_idx):
@@ -167,7 +175,7 @@ class ceVAE(pl.LightningModule):
         return loss
 
     def test_step(self, batch, batch_idx):
-        print("test step not defined")
+        logger.warn("test step not defined")
         pass
 
     def forward(self, x):
@@ -294,7 +302,7 @@ class ceVAE(pl.LightningModule):
 
         target_tensor = from_transforms(target_tensor[None])[0]
         # TODO rather normalize over the whole dataset rather than a single image
-        #save_image(target_tensor, file_name, normalize=True)
+        # save_image(target_tensor, file_name, normalize=True)
 
         return target_tensor
 
@@ -371,8 +379,8 @@ def main(
     logger.info("found config")
     logger.info(json.dumps(config, indent=2))
 
-    work_dir = Path(
-        config.log_dir.value) / Path(f"{datetime.datetime.now().strftime(folder_time_format)}_cevae")
+    current_run_name = f"{datetime.datetime.now().strftime(folder_time_format)}_cevae"
+    work_dir = Path(config.log_dir.value) / Path(current_run_name)
     if not os.path.exists(work_dir):
         os.makedirs(work_dir)
 
@@ -418,9 +426,8 @@ def main(
                 target_size=input_shape[2],
             )
         elif config.data.dataset.value == "SDOMLDatasetV1" or config.data.dataset.value == "SDOMLDatasetV2":
-
             if config.data.dataset.value == "SDOMLDatasetV1":
-                # due to a bug on Mac, num processes needs to be 0: https://github.com/pyg-team/pytorch_geometric/issues/366
+                # NOTE due to a bug on Mac, num_workers needs to be 0: https://github.com/pyg-team/pytorch_geometric/issues/366
                 data_module = SDOMLv1DataModule(base_dir=config.data.data_dir.value,
                                                 num_workers=config.data.num_data_loader_workers.value,
                                                 pin_memory=False,
@@ -434,39 +441,64 @@ def main(
                                                 pin_memory=config.data.pin_memory.value,
                                                 batch_size=config.data.batch_size.value,
                                                 prefetch_factor=config.data.prefetch_factor.value,
-                                                channel="171A",
-                                                year="2010",
+                                                channel=config.data.sdo_ml_v2.channel.value,
+                                                freq=config.data.sdo_ml_v2.freq.value,
+                                                irradiance=config.data.sdo_ml_v2.irradiance.value,
+                                                goes_cache_dir=config.data.sdo_ml_v2.goes_cache_dir.value,
                                                 target_size=input_shape[2],
+                                                train_year=config.data.sdo_ml_v2.train_year.value,
                                                 train_start=config.data.sdo_ml_v2.train_start_date.value,
                                                 train_end=config.data.sdo_ml_v2.train_end_date.value,
+                                                test_year=config.data.sdo_ml_v2.test_year.value,
                                                 test_start=config.data.sdo_ml_v2.test_start_date.value,
                                                 test_end=config.data.sdo_ml_v2.test_end_date.value,
                                                 train_val_split_ratio=config.data.sdo_ml_v2.train_val_split_ratio.value,
-                                                train_val_split_temporal_chunk_size=config.data.sdo_ml_v2.train_val_split_temporal_chunk_size.value)
+                                                train_val_split_temporal_chunk_size=config.data.sdo_ml_v2.train_val_split_temporal_chunk_size.value,
+                                                sampling_strategy=config.data.sdo_ml_v2.sampling_strategy.value,
+                                                mask_limb=config.data.sdo_ml_v2.mask_limb.value,
+                                                mask_limb_radius_scale_factor=config.data.sdo_ml_v2.mask_limb_radius_scale_factor.value)
 
-        wandb_logger = WandbLogger(project="sdo-sood", log_model="all")
+        # config_exclude_keys are already logged from config directly
+        wandb_logger = WandbLogger(
+            project="sdo-sood", log_model="all", config_exclude_keys=["input_shape",
+                                                                      "lr",
+                                                                      "z_dim",
+                                                                      "model_feature_map_sizes",
+                                                                      "use_geco",
+                                                                      "beta",
+                                                                      "ce_factor",
+                                                                      "score_mode",
+                                                                      "mode",
+                                                                      "print_every_iter"])
         wandb_logger.experiment.config.update(config)
 
-        from pytorch_lightning.profiler import PyTorchProfiler
+        profiler = None
+        if config.train.profile.value == True:
+            # https://pytorch-lightning.readthedocs.io/en/stable/advanced/profiler.html
+            # https://pytorch.org/tutorials/recipes/recipes/profiler_recipe.html
+            from pytorch_lightning.profiler import PyTorchProfiler
+            profiler = PyTorchProfiler(
+                #  dirpath=work_dir,
+                #  filename="output.profile",
+                record_shapes=True)
+
+        callbacks = [
+            ModelSummary(max_depth=4),
+            ModelCheckpoint(monitor="val_loss", dirpath=work_dir / Path("checkpoint"), filename="cevae-{epoch:02d}-{val_loss:.2f}")]
+        if config.train.early_stopping.value == True:
+            callbacks.append(EarlyStopping(
+                monitor="val_loss", mode="min", patience=10))
         trainer = pl.Trainer(logger=wandb_logger,
                              max_epochs=config.train.n_epochs.value,
+                             fast_dev_run=config.train.fast_dev_run.value,
                              # https://pytorch-lightning.readthedocs.io/en/1.4.7/common/single_gpu.html
                              # distributed training does not yet work because the data loader lambda cannot be pickled
-                             gpus=1,
-                             # TODO disable
-                             # https://pytorch-lightning.readthedocs.io/en/stable/advanced/profiler.html
-                             # https://pytorch.org/tutorials/recipes/recipes/profiler_recipe.html
-                             profiler=PyTorchProfiler(
-                                 #  dirpath=work_dir,
-                                 #  filename="output.profile",
-                                 record_shapes=True),
+                             gpus=config.devices.gpus.value,
+                             profiler=profiler,
+                             precision=32,
                              accelerator="auto",
                              default_root_dir=work_dir,
-                             callbacks=[
-                                 ModelSummary(max_depth=4),
-                                 EarlyStopping(
-                                     monitor="val_loss", mode="min", patience=10),
-                                 ModelCheckpoint(monitor="val_loss", dirpath=work_dir / Path("checkpoint"), filename="cevae-{epoch:02d}-{val_loss:.2f}")])
+                             callbacks=callbacks)
         # TODO log_graph does not work when running on multiple GPUs
         # # AttributeError: Can't pickle local object 'TorchGraph.create_forward_hook.<locals>.after_forward_hook'
         wandb_logger.watch(cevae_algo, log_graph=False)
@@ -483,8 +515,11 @@ def main(
         if pred_dir is None:
             pred_dir = work_dir / Path("predictions")
         elif pred_dir is None and work_dir is None:
-            print("Please either provide a log/output dir or a prediction dir")
+            logger.error(
+                "Please either provide a log/output dir or a prediction dir")
             sys.exit(0)
+        else:
+            pred_dir = Path(pred_dir) / Path(current_run_name)
 
         if not os.path.exists(pred_dir):
             os.makedirs(pred_dir, exist_ok=True)
@@ -513,17 +548,19 @@ def main(
                                                 storage_driver=config.data.sdo_ml_v2.storage_driver.value,
                                                 num_workers=config.data.num_data_loader_workers.value,
                                                 pin_memory=False,
+                                                target_size=input_shape[2],
                                                 batch_size=1,
                                                 prefetch_factor=config.data.prefetch_factor.value,
-                                                channel="171A",
-                                                year="2010",
-                                                target_size=input_shape[2],
-                                                train_start=config.data.sdo_ml_v2.train_start_date.value,
-                                                train_end=config.data.sdo_ml_v2.train_end_date.value,
+                                                channel=config.data.sdo_ml_v2.channel.value,
+                                                freq=config.data.sdo_ml_v2.freq.value,
+                                                irradiance=config.data.sdo_ml_v2.irradiance.value,
+                                                goes_cache_dir=config.data.sdo_ml_v2.goes_cache_dir.value,
+                                                test_year=config.data.sdo_ml_v2.test_year.value,
                                                 test_start=config.data.sdo_ml_v2.test_start_date.value,
                                                 test_end=config.data.sdo_ml_v2.test_end_date.value,
-                                                train_val_split_ratio=config.data.sdo_ml_v2.train_val_split_ratio.value,
-                                                train_val_split_temporal_chunk_size=config.data.sdo_ml_v2.train_val_split_temporal_chunk_size.value)
+                                                skip_train_val=True,
+                                                mask_limb=config.data.sdo_ml_v2.mask_limb.value,
+                                                mask_limb_radius_scale_factor=config.data.sdo_ml_v2.mask_limb_radius_scale_factor.value)
                 data_loader = data_module.predict_dataloader()
         for index, batch in tqdm(enumerate(data_loader)):
             # TODO attrs are not available for ImageParameterDataset
@@ -536,6 +573,12 @@ def main(
             if config.predict.mode.value == "pixel":
                 pixel_scores = cevae_algo.score_pixels(img[0])
                 # TODO rather normalize over the full dataset
+                save_src_image = False  # TODO make configurable
+                if save_src_image:
+                    src_file_name = Path(
+                        f"{timestamp.strftime(folder_time_format)}_{wavelength}A_src.png")
+                    src_file_path = Path(pred_dir) / src_file_name
+                    save_image(img[0], src_file_path)
                 save_image(pixel_scores, file_path, normalize=True)
             if config.predict.mode.value == "sample":
                 # TODO make it work for batches instead of single images
